@@ -621,7 +621,10 @@ mdns_avahi_browser_callback (AvahiServiceBrowser *b, AvahiIfIndex interface,
         AvahiLookupResultFlags flags, void* userdata)
 {
     mdns_finding    *mdns;
-    MDNS_SERVICE    service = (MDNS_SERVICE) userdata;
+    // Newer clang produces a warning about casting from void* to an enum.
+    // userdata is only set to values from the MDNS_SERVICE enum, so we can
+    // safely suppress this warning with an extra cast through intptr_t.
+    MDNS_SERVICE    service = (MDNS_SERVICE)(intptr_t) userdata;
     ZEROCONF_METHOD method = mdns_service_to_method(service);
     bool            initscan = mdns_initscan[service];
 
@@ -817,6 +820,9 @@ mdns_avahi_client_start (void)
 
     mdns_avahi_client = avahi_client_new (mdns_avahi_poll,
         AVAHI_CLIENT_NO_FAIL, mdns_avahi_client_callback, NULL, &error);
+    if (mdns_avahi_client == NULL) {
+	log_debug(mdns_log, "avahi_client_new failed: %s", avahi_strerror(error));
+    }
 }
 
 /* Deferred client restart
